from garbevents.custom_sensors_events import GetData
from garbevents.settings import Settings as ST


"gb -p 8889 -s test_custom_sensors_events.py"
ST.url = "https://sensors.wenwo.com"
ST.report_path = "report"
ST.all_events = ["$WebClick", "$pageview"]

addons = [GetData()]



import json
import os.path
import pathlib
import shutil
import time
import typing

from mitmproxy import contentviews
from mitmproxy import ctx
from mitmproxy import flow
from mitmproxy import flowfilter
from mitmproxy import http
from mitmproxy import io
from mitmproxy import version

from garbevents.web.app import flow_to_json

web_dir = pathlib.Path(__file__).absolute().parent


def save_static(path: pathlib.Path) -> None:
    """
    Save the files for the static web view.
    """
    # We want to overwrite the static files to keep track of the update.
    if (path / "static").exists():
        shutil.rmtree(str(path / "static"))
    shutil.copytree(str(web_dir / "static"), str(path / "static"))
    shutil.copyfile(str(web_dir / "templates" / "index.html"), str(path / "index.html"))

    with open(str(path / "static" / "static.js"), "w") as f:
        f.write("MITMWEB_STATIC = true;")


def save_filter_help(path: pathlib.Path) -> None:
    with open(str(path / "filter-help.json"), "w") as f:
        json.dump(dict(commands=flowfilter.help), f)


def save_settings(path: pathlib.Path) -> None:
    with open(str(path / "settings.json"), "w") as f:
        json.dump(dict(version=version.VERSION), f)


def save_flows(path: pathlib.Path, flows: typing.Iterable[flow.Flow]) -> None:
    with open(str(path / "flows.json"), "w") as f:
        json.dump([flow_to_json(f) for f in flows], f)


def save_flows_content(path: pathlib.Path, flows: typing.Iterable[flow.Flow]) -> None:
    for f in flows:
        assert isinstance(f, http.HTTPFlow)
        for m in ("request", "response"):
            message = getattr(f, m)
            message_path = path / "flows" / f.id / m
            os.makedirs(str(message_path / "content"), exist_ok=True)

            with open(str(message_path / "content.data"), "wb") as content_file:
                # don't use raw_content here as this is served with a default content type
                if message:
                    content_file.write(message.content)
                else:
                    content_file.write(b"No content.")

            # content_view
            t = time.time()
            if message:
                description, lines, error = contentviews.get_message_content_view(
                    "Auto", message, f
                )
            else:
                description, lines = "No content.", []
            if time.time() - t > 0.1:
                ctx.log(
                    "Slow content view: {} took {}s".format(
                        description.strip(), round(time.time() - t, 1)
                    ),
                    "info",
                )
            with open(
                str(message_path / "content" / "Auto.json"), "w"
            ) as content_view_file:
                json.dump(
                    dict(lines=list(lines), description=description), content_view_file
                )


class StaticViewer:
    # TODO: make this a command at some point.
    def load(self, loader):
        loader.add_option(
            "web_static_viewer",
            typing.Optional[str],
            "",
            "The path to output a static viewer.",
        )

    def configure(self, updated):
        if "web_static_viewer" in updated and ctx.options.web_static_viewer:
            flows = io.read_flows_from_paths([ctx.options.rfile])
            p = pathlib.Path(ctx.options.web_static_viewer).expanduser()
            self.export(p, flows)

    def export(self, path: pathlib.Path, flows: typing.Iterable[flow.Flow]) -> None:
        save_static(path)
        save_filter_help(path)
        save_flows(path, flows)
        save_flows_content(path, flows)



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2022/5/6 9:52 上午
@Desc    :  weak net line.
"""
import re
import time
import typing

from mitmproxy import ctx
from mitmproxy import exceptions
from mitmproxy import flowfilter
from mitmproxy.utils.spec import parse_spec

bandwidth_type = [
    {"template_name": "2G", "bandwidth": 10, "display": "2G (10 Kb/s)"},
    {"template_name": "2.5G", "bandwidth": 35, "display": "2.5G (35 Kb/s)"},
    {"template_name": "3G", "bandwidth": 120, "display": "3G (120 Kb/s)"},
]
indexes_by_name = {i["template_name"]: i["bandwidth"] for i in bandwidth_type}


class WeakNetsSpec(typing.NamedTuple):
    matches: flowfilter.TFilter
    subject: str
    bandwidth: str


def parse_weak_net_spec(option: str) -> WeakNetsSpec:
    """

    :param option:
    :return:
    """
    spec = WeakNetsSpec(*parse_spec(option))

    try:
        re.compile(spec.subject)
    except re.error as e:
        raise ValueError(f"Invalid regular expression {spec.subject!r} ({e})")

    if not indexes_by_name.get(spec.bandwidth):
        raise ValueError(f"Invalid bandwidth type {spec.bandwidth!r}")

    return spec


class WeakNets:
    def __init__(self):
        self.bandwidth_templates: list[WeakNetsSpec] = []

    def load(self, loader):
        loader.add_option(
            "weak_nets",
            typing.Sequence[str],
            [],
            """
            使用"|flow-filter|url-regex|bandwidth"形式的模式限制网络带宽，其中分隔符是|，支持模拟 2G,2.5G,3G 带宽。
            """,
        )

    def configure(self, updated):
        if "weak_nets" in updated:
            self.bandwidth_templates = []
            for option in ctx.options.weak_nets:
                try:
                    spec = parse_weak_net_spec(option)
                except ValueError as e:
                    raise exceptions.OptionsError(
                        f"Cannot parse weak_nets option {option}: {e}"
                    ) from e

                self.bandwidth_templates.append(spec)

    def request(self, flow):
        if flow.response or flow.error or not flow.live:
            return
        self.run(flow)

    def response(self, flow):
        if flow.error or not flow.live:
            return
        self.run(flow)

    def run(self, flow):
        for spec in self.bandwidth_templates:
            if spec.matches(flow):
                chunk_size = 2048
                if flow.response:
                    try:
                        length = len(flow.response.headers["Content-length"])
                    except (ValueError, UnicodeDecodeError, KeyError):
                        length = 0
                    sleep_time = chunk_size / (
                        indexes_by_name.get(spec.bandwidth) * 1024
                    )
                    for i in range(int(length / chunk_size) + 1):
                        time.sleep(sleep_time)
                else:
                    try:
                        length = len(flow.request.headers["Content-length"])
                    except (ValueError, UnicodeDecodeError, KeyError):
                        length = 0
                    sleep_time = chunk_size / (
                        indexes_by_name.get(spec.bandwidth) * 1024
                    )
                    for i in range(int(length / chunk_size) + 1):
                        time.sleep(sleep_time)



from garbevents.sensors_events import GetData
from garbevents.settings import Settings as ST

"gb -p 8889 -s test_sensors_events.py"
ST.url = "https://sensors.wenwo.com"
ST.report_path = "report"
ST.all_events = [
    "LX_CONVERSATION_MSG_ITEM_FORWAR_CLICK",
    "LX_CONVERSATION_SETTING_TOPS_CLICK",
]

addons = [GetData()]



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2022/3/21 10:35 上午
@Desc    :  capture line.
"""
import json
from mitmproxy import http
from loguru import logger

from garbevents.db import proxy_data
from garbevents.settings import Settings as ST
from garbevents.toolbox import timestamp


class GetData:
    """
    A garbevents HTTP engine class.
    """

    def __init__(self):
        self.sql = proxy_data
        self.sql.execute(
            "CREATE TABLE IF NOT EXISTS proxy(id INTEGER PRIMARY KEY AUTOINCREMENT,timestamps,api,request,response,client_ip);"
        )

    def response(self, flow: http.HTTPFlow):
        """
        Proxy service data analysis
        :param flow:
        :return:
        """

        if ST.url in flow.request.url:
            client_ip = flow.client_conn.peername[0]
            logger.info(
                "Get client IP address after splitting ====>" "{}".format(client_ip)
            )
            api = flow.request.path.split("?")[0]
            logger.info("Get API address after splitting ====>" "{}".format(api))
            try:
                request_content = json.loads(flow.request.get_text())
            except Exception as e:
                logger.error(e)
                request_content = {}
            logger.info(
                "Get request content after splitting ====>" "{}".format(request_content)
            )
            try:
                response_content = json.loads(flow.response.text)
            except Exception as e:
                logger.error(e)
                response_content = {}
            logger.info(
                "Get response content after splitting ====>"
                "{}".format(response_content)
            )
            self.sql.execute(
                "INSERT INTO proxy values(NULL,?,?,?,?,?)",
                (
                    timestamp(),
                    api,
                    str(request_content),
                    str(response_content),
                    str(client_ip),
                ),
            )



import webbrowser
from typing import Sequence
from mitmproxy import ctx


class WebAddon:
    def load(self, loader):
        loader.add_option("web_open_browser", bool, True, "Start a browser.")
        loader.add_option("web_debug", bool, False, "Enable mitmweb debugging.")
        loader.add_option("web_port", int, 8088, "Web UI port.")
        loader.add_option("web_host", str, "0.0.0.0", "Web UI host.")
        loader.add_option(
            "web_columns",
            Sequence[str],
            ["tls", "icon", "path", "method", "status", "size", "time"],
            "Columns to show in the flow list",
        )
        loader.add_option("beta_clean", bool, False, "Enable clean all button.")
        loader.add_option("beta_docs", bool, False, "Use custom docs url.")

    def running(self):
        if hasattr(ctx.options, "web_open_browser") and ctx.options.web_open_browser:
            web_url = f"http://{ctx.options.web_host}:{ctx.options.web_port}/"
            success = open_browser(web_url)
            if not success:
                ctx.log.info(
                    f"No web browser found. Please open a browser and point it to {web_url}",
                )


def open_browser(url: str) -> bool:
    """
    Open a URL in a browser window.
    In contrast to webbrowser.open, we limit the list of suitable browsers.
    This gracefully degrades to a no-op on headless servers, where webbrowser.open
    would otherwise open lynx.

    Returns:
        True, if a browser has been opened
        False, if no suitable browser has been found.
    """
    browsers = (
        "windows-default",
        "macosx",
        "wslview %s",
        "x-www-browser %s",
        "gnome-open %s",
        "xdg-open",
        "google-chrome",
        "chrome",
        "chromium",
        "chromium-browser",
        "firefox",
        "opera",
        "safari",
    )
    for browser in browsers:
        try:
            b = webbrowser.get(browser)
        except webbrowser.Error:
            pass
        else:
            if b.open(url):
                return True
    return False



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2022/7/12 7:48 下午
@Desc    :  mock events line.
"""
import re
import typing
from pathlib import Path

from mitmproxy import ctx
from mitmproxy import exceptions
from mitmproxy import flowfilter
from mitmproxy import http
from mitmproxy.utils import strutils
from mitmproxy.utils.spec import parse_spec


class MockEventsSpec(typing.NamedTuple):
    """
    |flow-filter|url-regex|replacement-json
    """
    matches: flowfilter.TFilter
    subject: str
    replacement_str: str


    def read_replacement(self) -> bytes:
        """
        Process the replacement str. This usually just involves converting it to bytes.
        However, if it starts with `@`, we interpret the rest as a file path to read from.

        Raises:
            - IOError if the file cannot be read.
        """
        if self.replacement_str.startswith("@"):
            return Path(self.replacement_str[1:]).expanduser().read_bytes()
        else:
            # We could cache this at some point, but unlikely to be a problem.
            return strutils.escaped_str_to_bytes(self.replacement_str)


def parse_mock_events_spec(option: str) -> MockEventsSpec:
    spec = MockEventsSpec(*parse_spec(option))

    try:
        re.compile(spec.subject)
    except re.error as e:
        raise ValueError(f"Invalid regular expression {spec.subject!r} ({e})")

    return spec


class MockEvents:
    def __init__(self):
        self.replacements: list[MockEventsSpec] = []

    def load(self, loader):
        loader.add_option(
            "mock_events",
            typing.Sequence[str],
            [],
            """
            使用"|flow-filter|url-regex|replacement-json"形式的模式将响应结果修改，其中分隔符是|。
            """,
        )

    def configure(self, updated):
        if "mock_events" in updated:
            self.replacements = []
            for option in ctx.options.mock_events:
                try:
                    spec = parse_mock_events_spec(option)
                except ValueError as e:
                    raise exceptions.OptionsError(
                        f"Cannot parse mock_events option {option}: {e}"
                    ) from e

                self.replacements.append(spec)

    def response(self, flow: http.HTTPFlow) -> None:
        if flow.error or not flow.live:
            return
        url = flow.request.pretty_url
        for spec in self.replacements:
            if spec.matches(flow) and re.search(spec.subject, url):
                try:
                    contents = spec.read_replacement()
                except (TypeError, KeyError):
                    contents = {}
                flow.response.content = contents



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  2024/5/10 7:29 PM
@Desc    :  modifybody line.
"""
import re
import typing

from mitmproxy import ctx, exceptions
from mitmproxy.addons.modifyheaders import parse_modify_spec, ModifySpec


class ModifyBodys:
    """
    https://docs.mitmproxy.org/dev/overview-features/#modify-body
    request: |~u url & ~s|.*|replacement
    response: |~u url & ~q|.*|replacement
    """
    def __init__(self):
        self.replacements: typing.List[ModifySpec] = []

    def load(self, loader):
        loader.add_option(
            "modify_bodys", typing.Sequence[str], [],
            """
            使用"|flow-filter|regex|[@]replacement-json"形式的替换模式，@允许提供用于读取替换字符串的文件路径，其中分隔符是|。
            """
        )

    def configure(self, updated):
        if "modify_bodys" in updated:
            self.replacements = []
            for option in ctx.options.modify_body:
                try:
                    spec = parse_modify_spec(option, True)
                except ValueError as e:
                    raise exceptions.OptionsError(f"Cannot parse modify_body option {option}: {e}") from e

                self.replacements.append(spec)

    def request(self, flow):
        if flow.response or flow.error or not flow.live:
            return
        self.run(flow)

    def response(self, flow):
        if flow.error or not flow.live:
            return
        self.run(flow)

    def run(self, flow):
        for spec in self.replacements:
            if spec.matches(flow):
                try:
                    replacement = spec.read_replacement()
                except OSError as e:
                    ctx.log.warn(f"Could not read replacement file: {e}")
                    continue
                if flow.response:
                    flow.response.content = re.sub(spec.subject, replacement, flow.response.content, flags=re.DOTALL)
                else:
                    flow.request.content = re.sub(spec.subject, replacement, flow.request.content, flags=re.DOTALL)



import gzip
import json
import os
import select
from collections import Counter
from datetime import datetime
from pprint import pprint

import jsonpath
from loguru import logger

try:
    import paramiko
except ImportError:
    logger.warning("paramiko not installed, please install paramiko first.")
from mitmproxy import ctx

from garbevents.settings import Settings as ST


class GetData:
    """
    Capture of buried points in the background.
    A engine HTTP request class.
    """

    events_list = []
    now_time = datetime.now()
    new_time = now_time.strftime("%Y-%m-%d %H:%M:%S")

    @staticmethod
    def chunks(arr, n):
        """

        :param arr:
        :param n:
        :return:
        """
        return [arr[i: i + n] for i in range(0, len(arr), n)]

    @staticmethod
    def gzip_decompress(data):
        """
        解密上报数据
        :param data:
        :return:
        """
        try:
            return gzip.decompress(data)
        except AttributeError:
            from io import StringIO

            buf = StringIO()
            buf.write(data)
            fd = gzip.GzipFile(fileobj=buf, mode="r")
            fd.rewind()
            value = fd.read()
            fd.close()
            return value

    @staticmethod
    def backstage(serverip, user, pwd):
        """

        :param serverip:
        :param user:
        :param pwd:
        :return:
        """
        events_list = []
        # 构造当天的日志文件名字，根据不同的埋点名称再做修改
        now_time = datetime.now()
        new_time = now_time.strftime("%Y-%m-%d")
        log = "sensors.log." + new_time

        # 进行连接
        ctx.log.info("------------开始连接服务器(%s)-----------" % serverip)
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ctx.log.info("------------开始认证......-----------")
        client.connect(serverip, 22, username=user, password=pwd, timeout=4)
        ctx.log.info("------------认证成功!.....-----------")
        # 开启channel 管道
        transport = client.get_transport()
        channel = transport.open_session()
        channel.get_pty()
        # 执行命令nohup.log.2017-12-30
        tail = "tail -n 0 -f  " + ST.url + log
        # 将命令传入管道中
        channel.exec_command(tail)
        while True:
            # 用于精确查出什么时间时候没有值
            news_time = now_time.strftime("%Y-%m-%d %H:%M:%S")

            # 判断退出的准备状态
            if channel.exit_status_ready():
                break
            try:
                # 通过socket进行读取日志，linux相当于客户端
                rl, wl, el = select.select([channel], [], [])
                if len(rl) > 0:
                    recv = channel.recv(1024)
                    # 此处将获取的数据解码成utf-8的读取
                    AAA = recv.decode("utf-8", "ignore")
                    BBB = AAA.split("\n")
                    DDD = BBB[0:2]
                    for CCC in DDD:
                        try:
                            GGG = str(CCC)
                            data = json.loads(GGG)
                            try:
                                event = data["event"]
                                if event in ST.all_events:
                                    ctx.log.info(f"【后端】获取到的埋点事件名称是 ====>{event}")
                                    events_list.append(event)
                                    count = Counter(events_list)
                                    lost_list = list(
                                        set(ST.all_events).difference(set(events_list))
                                    )

                                    # 用于遍历对应event事件里面的具体信息
                                    for event_value in ST.events_properties[event]:
                                        # 记录下坑，下面不能直接在里面传event_value不然取不到值
                                        result = jsonpath.jsonpath(
                                            data, f"$..{event_value}"
                                        )
                                        if result is False:
                                            ctx.log.info(
                                                f"【后端】请注意！！！请注意！！！{event}事件未获取到{event_value}值"
                                            )
                                            with open(
                                                    f"{ST.report_path}/【后端】埋点事件参数没有值的统计.txt",
                                                    "a+",
                                                    encoding="utf-8",
                                            ) as file01:
                                                # 用于记录埋点事件参数没有值的统计.txt
                                                file01.write(
                                                    "【后端】{}请注意！！！请注意！！！{}事件未获取到{}值!!!".format(
                                                        news_time, event, event_value
                                                    )
                                                    + "\n"
                                                )
                                        else:
                                            ctx.log.info(
                                                f"【后端】获取到的埋点事件:{event}的自定义参数{event_value}的值是{result}："
                                            )

                                    # 在已上报的埋点事件统计.txt记录已经获取到的事件
                                    with open(
                                            f"{ST.report_path}/【后端】已上报的埋点事件统计.txt",
                                            "w",
                                            encoding="utf-8",
                                    ) as file03:
                                        # file03.writelines(events_list)
                                        for line in events_list:
                                            file03.write(line + "\n")

                                    # 在漏上报的埋点事件统计.txt记录已经获取到的事件
                                    with open(
                                            f"{ST.report_path}/【后端】漏上报的埋点事件统计.txt",
                                            "w",
                                            encoding="utf-8",
                                    ) as file04:
                                        # file04.writelines(lost_list)
                                        for lines in lost_list:
                                            file04.write(lines + "\n")
                                            ctx.log.info(
                                                f"【后端】暂未获取到这些埋点事件，可能存在漏上报事件 ====>{lines}"
                                            )

                                    ctx.log.info(f"【后端】累计的埋点事件统计次数{count}")
                                    with open(
                                            f"{ST.report_path}/【后端】累计的埋点事件统计次数.txt",
                                            "w",
                                            encoding="utf-8",
                                    ) as file02:
                                        # 用于记录埋点事件参数没有值的统计.txt
                                        file02.write(f"【后端】累计统计到的埋点次数{count}!!!" + "\n")

                                    # 不存在对应txt文件时，先创建txt文件
                                    if not os.path.exists(ST.report_path):
                                        os.mkdir(ST.report_path)
                                        ctx.log.info(
                                            ST.report_path + "Successfully created！"
                                        )
                                else:
                                    ctx.log.info(f"该事件{event}不在你录的事件中")
                            except Exception:
                                ctx.log.info("json中不存在event字段,请查看上面具体json查询想要的内容字段")
                                pprint(f"json中不存在event字段{data}")
                                break
                        except json.decoder.JSONDecodeError as e01:
                            pass
                        except Exception as e02:
                            print(e02)
                            ctx.log.info("读取不到日志文件，还未生成，请先手动触发事件生成日志")

            # 键盘终端异常
            except KeyboardInterrupt:
                print("Caught control-C")
                channel.send("\x03")  # 发送 ctrl+c
                channel.close()
        client.close()



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  2024/5/10 7:29 PM
@Desc    :  modifybody line.
"""
import json
from loguru import logger


class ModifyRules:
    """
    mj = ModifyRules()
    data = mj.get_json_value(test_json, 'location')
    pprint(data)

    mj.set_json_value(test_json, 'name', '北京市')
    pprint(test_json)

    mj.set_json_value_batch(test_json, {'name': '北京市'}, {'location': '北京市海淀区'})
    pprint(test_json)

    data = mj.replace_str_value(test_json, {'name': '北京市'}, {'location': '北京市海淀区'})
    pprint(data)
    """
    def get_json_value(self, json_data, key, json_list=None):
        """
        Recursively iterate the values corresponding to all keys in JSON, obtain the value from the key value, and output it as a list
        :param json_data:
        :param key:
        :param json_list:
        :return: json_list
        """
        # noinspection PyBroadException
        if json_list is None:
            json_list=[]
        # noinspection PyBroadException
        try:
            # 传入数据存在则存入json_list
            if key in json_data.keys():
                json_list.append(json_data[key])
            # 传入数据不符合则对其value值进行遍历
            for value in json_data.values():
                # 传入数据的value值是字典，则直接调用自身
                if isinstance(value, dict):
                    self.get_json_value(value, key, json_list)
                # 传入数据的value值是列表或者元组，则调用get_value
                elif isinstance(value, (list, tuple)):
                    self.get_value(value, key, json_list)
            return json_list
        except BaseException as e:
            logger.error('get_json_value function: {}'.format(e))

    def get_value(self, json_data, key, json_list):
        """
        Submethod: Recursively iterate the value corresponding to all keys in JSON, obtain the value from the key value, and output it as a list
        :param json_data:
        :param key:
        :param json_list:
        :return: None
        """
        for val in json_data:
            # 传入数据的value值是字典，则调用get_json_value
            if isinstance(val, dict):
                self.get_json_value(val, key, json_list)
            # 传入数据的value值是列表或者元组，则调用自身
            elif isinstance(val, (list, tuple)):
                self.get_value(val, key, json_list)

    def set_json_value(self, json_data, key, target_value):
        """
        Recursively iterate through the values corresponding to all keys in JSON and modify the values by the key values
        Only a single key is supported, and the value corresponding to all the same keys will be modified
        :param json_data:
        :param key:
        :param target_value:
        :return: json_data
        """
        # noinspection PyBroadException
        try:
            if isinstance(json_data, str):
                json_data=json.loads(json_data)
            # 传入数据存在则修改字典
            if key in json_data.keys():
                json_data[key]=target_value
            # 传入数据不符合则对其value值进行遍历
            for value in json_data.values():
                # 传入数据的value值是字典，则直接调用自身,将value作为字典传进来
                if isinstance(value, dict):
                    self.set_json_value(value, key, target_value)
                    value[key]=target_value
                # 传入数据的value值是列表或者元组，则调用set_value
                elif isinstance(value, (list, tuple)):
                    self.set_value(value, key, target_value)
            return json_data
        except BaseException as e:
            logger.error('set_json_value function: {}'.format(e))

    def set_value(self, json_data, key, target_value):
        """
        Sub-method: Recursively iterate the value corresponding to all keys in JSON, and modify the value by the key value
        :param json_data:
        :param key:
        :param target_value:
        :return: None
        """
        for val in json_data:
            # 传入数据的value值是字典，则调用get_json_value
            if isinstance(val, dict):
                self.set_json_value(val, key, target_value)
            # 传入数据的value值是列表或者元组，则调用自身
            elif isinstance(val, (list, tuple)):
                self.set_value(val, key, target_value)

    def set_json_value_batch(self, json_data, *args):
        """
        Multiple keys are supported, and the value corresponding to all the same keys will be modified
        :param json_data:
        :param args:
        :return: new_json_data
        """
        # noinspection PyBroadException
        try:
            if isinstance(args, tuple):
                if args == ():
                    return json_data
                else:
                    new_json_data=json_data
                    for arg in args:
                        for key in arg:
                            value=arg[key]
                            new_json_data=self.set_json_value(new_json_data, key, value)
                    return new_json_data
        except BaseException as e:
            logger.error('set_json_value_batch function: {}'.format(e))

    @staticmethod
    def replace_str_value(input_data, *args):
        """
        Batch replacement of values containing ${vars}, *args is dictionary format data, multiple parameters can be passed,
        suitable for request bodies in any format (json, xml, etc.)
        This function does not use self-related variables, so make this function a static method (brute force substitution)
        :param input_data:
        :param args:
        :return: new_data
        """
        # noinspection PyBroadException
        try:
            # 将传入数据强转为字符串进行替换
            new_data=str(input_data)
            for arg in args:
                for i in arg:
                    target_value=arg[i]
                    replace_value="${" + i + "}"
                    print(replace_value)
                    new_data=new_data.replace(replace_value, target_value)
            # 判断传进的数据如果为json格式,如果是就转换为json格式
            if isinstance(input_data, (list, dict)):
                return eval(new_data)
            return new_data
        except BaseException as e:
            logger.error('replace_str_value function: {}'.format(e))



"""
Parameter initialization is a global variable by default. When calling the relevant API,
you need to inherit the setting class and set the corresponding parameters.

"""


class Settings:
    """
    Settings class
    """

    report_path = "report"
    db_path = "proxy"
    url = None
    all_events = []
    interface_url = []
    mock = {"api": "", "content": ""}
    events_properties = {}
    result = None
    result_dict = None
    expect = None
    actual = None
    exclude_paths = None
    global_url = None
    api_collection = {}
    web_host: str = "0.0.0.0"
    web_port: int = 8088
    server_port: int = 8888
    # options command: map_remotes, map_locals, modify_bodys, modify_headers, mock_events
    options = "mock_events"
